@@ -9,11 +9,12 @@
 # This code is under the GNU General Public License.  See COPYING for details.
 
 from __future__ import print_function
-__version__ = '1.0.4'
+__version__ = '1.0.5'
 
 import os
 import re
 import milter
+import sys
 try:
   import thread
 except:
@@ -180,15 +181,41 @@ def noreply(func):
   wrapper.milter_protocol = nr_mask
   return wrapper
 
+## Function decorator to set decoding error strategy.
+# Current RFCs define UTF-8 as the standard encoding for SMTP
+# envelope and header fields.  By default, Milter.Base decodes 
+# envelope and header values with errors='surrogateescape'.
+# Applications can recover the original bytes with 
+# <pre>
+# b = s.encode(errors='surrogateescape')
+# </pre>
+# This preserves information, but can lead to unexpected exceptions
+# as you cannot, e.g. print strings with surrogates.
+# Illegal bytes occur quite often in real life, so there must
+# be a way to deal with them.
+# This decorator can change the error strategy to
+# <ul>
+# <li> bytes    - original bytes are passed unmodified
+# <li> strict   - pass bytes if illegal bytes are present, string otherwise
+# <li> ignore   - illegal bytes are removed
+# <li> replace  - illegal bytes are replaced with a unicode error symbol
+# </ul> 
+# 
+def decode(strategy):
+  def setstrategy(func):
+    func.error_strategy = strategy
+    return func
+  return setstrategy
+
 ## Function decorator to set macros used in a callback.
 # By default, the MTA sends all macros defined for a callback.
 # If some or all of these are unused, the bandwidth can be saved
 # by listing the ones that are used.
 # @since 1.0.2
 def symlist(*syms):
-  if len(syms) > 5:
-    raise ValueError('@symlist limited to 5 macros by MTA: '+func.__name__)
   def setsyms(func):
+    if len(syms) > 5:
+      raise ValueError('@symlist limited to 5 macros by MTA: '+func.__name__)
     if func.__name__ not in MACRO_CALLBACKS:
       raise ValueError('@symlist applied to non-symlist method: '+func.__name__)
     func._symlist = syms
@@ -320,6 +347,20 @@ class Base(object):
   # this almost always results in terminating the connection.
   @nocallback
   def hello(self,hostname): return CONTINUE
+  ## Called with bytes by default global envfrom callback.
+  # @since 1.0.5
+  # Converts from utf-8 to unicode with surrogate escape.  Can be overriden 
+  # to pass bytes to @link #header the header callback @endlink instead,
+  # or trap utf-8 conversion exception, etc.
+  def envfrom_bytes(self,*b):
+    try:
+      e = getattr(self.envfrom,'error_strategy','surrogateescape')
+      if e == 'bytes':
+        #self.envfrom_bytes = self.envfrom
+        return self.envfrom(*b)
+      s = [v.decode(encoding='utf-8',errors=e) for v in b]
+    except UnicodeDecodeError: s = b
+    return self.envfrom(s[0],*s[1:])
   ## Called when the SMTP client says MAIL FROM. Called by the
   # <a href="milter_api/xxfi_envfrom.html">
   # xxfi_envfrom</a> callback.  
@@ -330,7 +371,21 @@ class Base(object):
   # <a href="http://tools.ietf.org/html/rfc5322">RFC 5322</a>,
   # see @link #header the header callback @endlink.
   @nocallback
-  def envfrom(self,f,*str): return CONTINUE
+  def envfrom(self,f,*s): return CONTINUE
+  ## Called with bytes by default global envrcpt callback.
+  # @since 1.0.5
+  # Converts from utf-8 to unicode with surrogate escape.  Can be overriden 
+  # to pass bytes to @link #header the header callback @endlink instead,
+  # or trap utf-8 conversion exception, etc.
+  def envrcpt_bytes(self,*b):
+    try:
+      e = getattr(self.envrcpt,'error_strategy','surrogateescape')
+      if e == 'bytes':
+        #self.envrcpt_bytes = self.envrcpt
+        return self.envrcpt(*b)
+      s = [v.decode(encoding='utf-8',errors=e) for v in b]
+    except UnicodeDecodeError: s = b
+    return self.envrcpt(s[0],*s[1:])
   ## Called when the SMTP client says RCPT TO. Called by the
   # <a href="milter_api/xxfi_envrcpt.html">
   # xxfi_envrcpt</a> callback.
@@ -348,7 +403,30 @@ class Base(object):
   # @since 0.9.2
   @nocallback
   def data(self): return CONTINUE
+  ## Called with bytes by default global header callback.
+  # @param fld name decoded as ascii
+  # @param val field value as bytes
+  # @since 1.0.5
+  # Converts from utf-8 to unicode with surrogate escape.  Can be overriden 
+  # to pass bytes to @link #header the header callback @endlink instead,
+  # e.g. by assignment:
+  # <pre>
+  #  mymilter.header_bytes = mymilter.header
+  # </pre>
+  # The <code>@decode('bytes')</code> decorator will also do this.
+  # 
+  def header_bytes(self,fld,val):
+    try:
+      e = getattr(self.header,'error_strategy','surrogateescape')
+      if e == 'bytes':
+        self.header_bytes = self.header
+        return self.header(fld,val)
+      s = val.decode(encoding='utf-8',errors=e)
+    except UnicodeDecodeError: s = val
+    return self.header(fld,s)
   ## Called for each header field in the message body.
+  # @param field name decoded as ascii
+  # @param value field value decoded as utf-8 on python3
   @nocallback
   def header(self,field,value): return CONTINUE
   ## Called at the blank line that terminates the header fields.
@@ -764,9 +842,14 @@ def runmilter(name,socketname,timeout = 0,rmsock=True):
   # parms, but then all existing users would have to include **kw to accept
   # arbitrary keywords without crashing.  We do provide envcallback and
   # dictfromlist to make parsing the ESMTP args convenient.
-  milter.set_envfrom_callback(lambda ctx,*str: ctx.getpriv().envfrom(*str))
-  milter.set_envrcpt_callback(lambda ctx,*str: ctx.getpriv().envrcpt(*str))
-  milter.set_header_callback(lambda ctx,fld,val: ctx.getpriv().header(fld,val))
+  if sys.version < '3.0.0':
+    milter.set_envfrom_callback(lambda ctx,*s: ctx.getpriv().envfrom(*s))
+    milter.set_envrcpt_callback(lambda ctx,*s: ctx.getpriv().envrcpt(*s))
+    milter.set_header_callback(lambda ctx,f,v: ctx.getpriv().header(f,v))
+  else:
+    milter.set_envfrom_callback(lambda ctx,*b: ctx.getpriv().envfrom_bytes(*b))
+    milter.set_envrcpt_callback(lambda ctx,*b: ctx.getpriv().envrcpt_bytes(*b))
+    milter.set_header_callback(lambda ctx,f,v: ctx.getpriv().header_bytes(f,v))
   milter.set_eoh_callback(lambda ctx: ctx.getpriv().eoh())
   milter.set_body_callback(lambda ctx,chunk: ctx.getpriv().body(chunk))
   milter.set_eom_callback(lambda ctx: ctx.getpriv().eom())
